﻿using System.Collections;
using System.Collections.Generic;
using BepInEx;
using HarmonyLib;
using UnityEngine;
using UnityEngine.Networking;

namespace MuzikaGromche
{
    [BepInPlugin(PluginInfo.PLUGIN_GUID, PluginInfo.PLUGIN_NAME, PluginInfo.PLUGIN_VERSION)]
    public class Plugin : BaseUnityPlugin
    {
		public static AudioClip AudioStart;
		public static AudioClip AudioLoop;

		public static Coroutine JesterLightSwitching;

		private void Awake()
		{
			string text = Info.Location.TrimEnd((PluginInfo.PLUGIN_NAME + ".dll").ToCharArray());
			UnityWebRequest audioClipStart = UnityWebRequestMultimedia.GetAudioClip("File://" + text + "MuzikaGromcheStart.mp3", AudioType.MPEG);
			UnityWebRequest audioClipLoop = UnityWebRequestMultimedia.GetAudioClip("File://" + text + "MuzikaGromcheLoop.mp3", AudioType.MPEG);
			audioClipStart.SendWebRequest();
			audioClipLoop.SendWebRequest();
			while (!audioClipStart.isDone || !audioClipLoop.isDone) { }
			if (audioClipStart.result == UnityWebRequest.Result.Success && audioClipLoop.result == UnityWebRequest.Result.Success) {
				AudioStart = DownloadHandlerAudioClip.GetContent(audioClipStart);
				AudioLoop = DownloadHandlerAudioClip.GetContent(audioClipLoop);
				new Harmony(PluginInfo.PLUGIN_NAME).PatchAll(typeof(JesterPatch));
				Logger.LogInfo("Muzika gromche, glaza zakryvaj! " + PluginInfo.PLUGIN_VERSION);
			} else {
				Logger.LogError("Could not load audio file");
			}
		}
    }

    [HarmonyPatch(typeof(JesterAI))]
    internal class JesterPatch
    {
	    [HarmonyPatch("Start")]
	    [HarmonyPostfix]
	    public static void AudioPatch(JesterAI __instance)
	    {
		    __instance.popGoesTheWeaselTheme = Plugin.AudioStart;
		    __instance.screamingSFX = Plugin.AudioLoop;
	    }

	    [HarmonyPatch("SetJesterInitialValues")]
	    [HarmonyPostfix]
	    public static void ForceTime(JesterAI __instance)
	    {
		    __instance.popUpTimer = 46.3f;
	    }

	    [HarmonyPatch("Update")]
	    [HarmonyPrefix]
	    public static void DoNotStopTheMusicPrefix(JesterAI __instance, out State __state)
	    {
		    __state = new State();
		    __state.prevStateindex = __instance.previousState;
		    if (__instance.currentBehaviourStateIndex is 1 or 2) {
			    __state.farAudio = __instance.farAudio;
			    __instance.farAudio = __instance.creatureSFX;
		    }
	    }
	    
	    static List<Color> colors = [Color.magenta, Color.cyan, Color.green, Color.yellow];

	    public static IEnumerator rotateColors()
	    {
		    Debug.Log("Starting color rotation");
		    var i = 0;
		    while (true)
		    {
			    var color = colors[i];
			    Debug.Log("Chose color " + color);
			    foreach (var light in RoundManager.Instance.allPoweredLights)
			    {
				    light.color = color;
			    }

			    i += 1;
			    if (i >= colors.Count) i = 0;
			    yield return new WaitForSeconds(0.5f);
		    }
	    }

		[HarmonyPatch("Update")]
        [HarmonyPostfix]
        public static void DoNotStopTheMusic(JesterAI __instance, State __state)
        {
	        if (__state.farAudio != null)
	        {
		        __instance.farAudio = __state.farAudio;
	        }

	        if (__instance.currentBehaviourStateIndex is 1 && !__instance.farAudio.isPlaying)
	        {
		        Debug.Log($"Playing start music: maxDistance: {__instance.farAudio.maxDistance}, minDistance: {__instance.farAudio.minDistance}, volume: {__instance.farAudio.volume}, spread: {__instance.farAudio.spread}");
		        __instance.farAudio.maxDistance = 150;
		        __instance.farAudio.PlayOneShot(Plugin.AudioStart);
	        }

	        if (__instance.currentBehaviourStateIndex is 2 && __state.prevStateindex != 2)
	        {
		        __instance.creatureVoice.Stop();
		        
		        if (Plugin.JesterLightSwitching != null) {
			        __instance.StopCoroutine(Plugin.JesterLightSwitching);
			        Plugin.JesterLightSwitching = null;
		        }
		        

		        Plugin.JesterLightSwitching = __instance.StartCoroutine(rotateColors());
	        }

	        if (__instance.currentBehaviourStateIndex != 2 && __state.prevStateindex == 2)
	        {
		        if (Plugin.JesterLightSwitching != null) {
			        __instance.StopCoroutine(Plugin.JesterLightSwitching);
			        Plugin.JesterLightSwitching = null;
		        }
		        foreach (var light in RoundManager.Instance.allPoweredLights)
		        {
			        light.color = Color.white;
		        }
	        }

	        if (__instance.currentBehaviourStateIndex is 2 && !__instance.creatureVoice.isPlaying && !__instance.farAudio.isPlaying)
	        {
		        Debug.Log($"Playing loop music: maxDistance: {__instance.creatureVoice.maxDistance}, minDistance: {__instance.creatureVoice.minDistance}, volume: {__instance.creatureVoice.volume}, spread: {__instance.creatureVoice.spread}");
		        __instance.creatureVoice.maxDistance = 150;
		        __instance.creatureVoice.clip = Plugin.AudioLoop;
		        __instance.creatureVoice.Play();
	        }
        }
    }

    internal class State
    {
	    public AudioSource farAudio;
	    public int prevStateindex;
    }
}
